@@ -68,7 +68,8 @@ Q_SIGNALS:
     void listCategoryInternal(Category *cat);
 
     void unableToFind(const QString &resid);
-    void openErrorPage(const QString &errorMessage, const QString &errorExplanation, const QString &buttonText, const QString &buttonIcon, const QString &buttonURL);
+    void
+    openErrorPage(const QString &errorMessage, const QString &errorExplanation, const QString &buttonText, const QString &buttonIcon, const QString &buttonURL);
 
 private:
     void showLoadingPage();
