@@ -4,33 +4,33 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef APPSTREAMINTEGRATION_H
-#define APPSTREAMINTEGRATION_H
+#pragma once
 
-#include "OdrsReviewsBackend.h"
 #include "discovercommon_export.h"
+
+#include <AppStreamQt/pool.h>
+#include <AppStreamQt/release.h>
 #include <KOSRelease>
 #include <QObject>
 
+#include <optional>
+
 class DISCOVERCOMMON_EXPORT AppStreamIntegration : public QObject
 {
     Q_OBJECT
 public:
     static AppStreamIntegration *global();
 
-    QSharedPointer<OdrsReviewsBackend> reviews();
+    std::optional<AppStream::Release> getDistroUpgrade(AppStream::Pool *pool);
     KOSRelease *osRelease()
     {
         return &m_osrelease;
     }
 
 private:
-    QWeakPointer<OdrsReviewsBackend> m_reviews;
     KOSRelease m_osrelease;
 
     AppStreamIntegration()
     {
     }
 };
-
-#endif // APPSTREAMINTEGRATION_H
