@@ -10,6 +10,7 @@
 #include "FlatpakFetchDataJob.h"
 #include "FlatpakSourcesBackend.h"
 #include "config-paths.h"
+#include "libdiscover_backend_flatpak_debug.h"
 
 #include <Transaction/AddonList.h>
 
@@ -28,7 +29,7 @@
 #include <KLocalizedString>
 
 #include <AppStreamQt/release.h>
-#include <QDebug>
+#include <QCoroCore>
 #include <QDesktopServices>
 #include <QDir>
 #include <QDirIterator>
@@ -77,35 +78,51 @@ FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakI
 
     // Start fetching remote icons during initialization
     const auto icons = m_appdata.icons();
-    if (icons.count() == 1 && icons.constFirst().kind() == AppStream::Icon::KindRemote) {
-        const auto icon = icons.constFirst();
-        const QString fileName = iconCachePath(icon);
-        if (!QFileInfo::exists(fileName)) {
-            const QDir cacheDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
-            // Create $HOME/.cache/discover/icons folder
-            cacheDir.mkdir(QStringLiteral("icons"));
-            auto reply = manager->get(QNetworkRequest(icon.url()));
-            connect(reply, &QNetworkReply::finished, this, [this, icon, fileName, reply] {
-                if (reply->error() == QNetworkReply::NoError) {
-                    QByteArray iconData = reply->readAll();
-                    QFile file(fileName);
-                    if (file.open(QIODevice::WriteOnly)) {
-                        file.write(iconData);
-                    } else {
-                        qDebug() << "could not find icon for" << packageName() << reply->url();
-                        QIcon::fromTheme(QStringLiteral("package-x-generic")).pixmap(32, 32).toImage().save(fileName);
+    m_stockIcon = std::ranges::any_of(icons, [](const AppStream::Icon &icon) {
+        return icon.kind() == AppStream::Icon::KindStock && KIconLoader::global()->hasIcon(icon.name());
+    });
+    if (!m_stockIcon && !icons.isEmpty() && !std::ranges::any_of(icons, [](const AppStream::Icon &icon) {
+            return icon.kind() == AppStream::Icon::KindLocal || icon.kind() == AppStream::Icon::KindCached;
+        })) {
+        for (const auto &icon : icons) {
+            if (icon.kind() != AppStream::Icon::KindRemote) {
+                continue;
+            }
+            const QString fileName = iconCachePath(icon);
+            if (!QFileInfo::exists(fileName)) {
+                const QDir cacheDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
+                // Create $HOME/.cache/discover/icons folder
+                cacheDir.mkdir(QStringLiteral("icons"));
+                auto reply = manager->get(QNetworkRequest(icon.url()));
+                connect(reply, &QNetworkReply::finished, this, [this, icon, fileName, reply] {
+                    if (reply->error() == QNetworkReply::NoError) {
+                        QByteArray iconData = reply->readAll();
+                        QFile file(fileName);
+                        if (file.open(QIODevice::WriteOnly)) {
+                            file.write(iconData);
+                        } else {
+                            qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "could not find icon for" << packageName() << reply->url();
+                            QIcon::fromTheme(QStringLiteral("package-x-generic")).pixmap(32, 32).toImage().save(fileName);
+                        }
+                        file.close();
+                        Q_EMIT iconChanged();
+                        reply->deleteLater();
                     }
-                    file.close();
-                    Q_EMIT iconChanged();
-                    reply->deleteLater();
-                }
-            });
+                });
+            }
         }
     }
 
     connect(this, &FlatpakResource::stateChanged, this, &FlatpakResource::hasDataChanged);
 }
 
+FlatpakBackend *FlatpakResource::backend() const
+{
+    const auto backend = qobject_cast<FlatpakBackend *>(AbstractResource::backend());
+    Q_ASSERT(backend);
+    return backend;
+}
+
 AppStream::Component FlatpakResource::appstreamComponent() const
 {
     return m_appdata;
@@ -164,7 +181,6 @@ void FlatpakResource::updateFromRef(FlatpakRef *ref)
 {
     setArch(QString::fromUtf8(flatpak_ref_get_arch(ref)));
     setBranch(QString::fromUtf8(flatpak_ref_get_branch(ref)));
-    setCommit(QString::fromUtf8(flatpak_ref_get_commit(ref)));
     setFlatpakName(QString::fromUtf8(flatpak_ref_get_name(ref)));
     setType(flatpak_ref_get_kind(ref) == FLATPAK_REF_KIND_APP ? DesktopApp : extends().isEmpty() ? Runtime : Extension);
     setObjectName(packageName());
@@ -176,7 +192,7 @@ void FlatpakResource::updateFromAppStream()
     g_autoptr(GError) localError = nullptr;
     g_autoptr(FlatpakRef) ref = flatpak_ref_parse(refstr.toUtf8().constData(), &localError);
     if (!ref) {
-        qDebug() << "failed to obtain ref" << refstr << localError->message;
+        qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "failed to obtain ref" << refstr << localError->message;
         return;
     }
     updateFromRef(ref);
@@ -200,11 +216,6 @@ QString FlatpakResource::comment()
     return QString();
 }
 
-QString FlatpakResource::commit() const
-{
-    return m_commit;
-}
-
 quint64 FlatpakResource::downloadSize() const
 {
     return m_downloadSize;
@@ -240,7 +251,7 @@ QVariant FlatpakResource::icon() const
                 }
             } break;
             case AppStream::Icon::KindStock: {
-                if (KIconLoader::global()->hasIcon(icon.name())) {
+                if (m_stockIcon) {
                     return QIcon::fromTheme(icon.name());
                 }
                 break;
@@ -266,7 +277,7 @@ QVariant FlatpakResource::icon() const
 
 QString FlatpakResource::installedVersion() const
 {
-    g_autoptr(FlatpakInstalledRef) ref = qobject_cast<FlatpakBackend *>(backend())->getInstalledRefForApp(this);
+    g_autoptr(FlatpakInstalledRef) ref = backend()->getInstalledRefForApp(this);
     if (ref) {
         const char *appdataVersion = flatpak_installed_ref_get_appdata_version(ref);
 
@@ -398,17 +409,17 @@ QString FlatpakResource::displayOrigin() const
 // use string builder with arguments directly instead of temporary allocated strings.
 QString FlatpakResource::packageName() const
 {
-    return QStringLiteral("%1/%2/%3").arg(flatpakName(), arch(), branch());
+    return flatpakName() + '/'_L1 + arch() + '/'_L1 + branch();
 }
 
 QString FlatpakResource::ref() const
 {
-    return QStringLiteral("%1/%2/%3/%4").arg(typeAsString(), flatpakName(), arch(), branch());
+    return typeAsString() + '/'_L1 + flatpakName() + '/'_L1 + arch() + '/'_L1 + branch();
 }
 
 QString FlatpakResource::installPath() const
 {
-    return QStringLiteral("%1/%2/%3/%4/%5/active").arg(installationPath(), typeAsString(), flatpakName(), arch(), branch());
+    return installationPath() + '/'_L1 + typeAsString() + '/'_L1 + flatpakName() + '/'_L1 + arch() + '/'_L1 + branch() + "/active"_L1;
 }
 
 FlatpakResource::PropertyState FlatpakResource::propertyState(FlatpakResource::PropertyKind kind) const
@@ -443,7 +454,7 @@ quint64 FlatpakResource::size()
 QString FlatpakResource::sizeDescription()
 {
     if (propertyState(InstalledSize) == NotKnownYet || propertyState(InstalledSize) == Fetching) {
-        qobject_cast<FlatpakBackend *>(backend())->updateAppSize(this);
+        backend()->updateAppSize(this);
         return i18n("Retrieving size information");
     } else if (propertyState(InstalledSize) == UnknownOrFailed) {
         return i18nc("@label app size", "Unknown");
@@ -463,7 +474,7 @@ FlatpakResource::ResourceType FlatpakResource::resourceType() const
     return m_type;
 }
 
-QString FlatpakResource::typeAsString() const
+QLatin1String FlatpakResource::typeAsString() const
 {
     switch (m_type) {
     case FlatpakResource::Runtime:
@@ -488,18 +499,18 @@ void FlatpakResource::invokeApplication() const
     if (!launchables.isEmpty()) {
         desktopFileName = launchables.constFirst();
     } else {
-        qWarning() << "Failed to find launchable for " << m_appdata.name() << ", using AppStream identifier instead";
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to find launchable for " << m_appdata.name() << ", using AppStream identifier instead";
         desktopFileName = appstreamId();
     }
 
     KService::Ptr service = KService::serviceByStorageId(desktopFileName);
 
     if (!service) {
-        qWarning() << "Failed to find service" << desktopFileName;
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to find service" << desktopFileName;
         return;
     }
 
-    auto *job = new KIO::ApplicationLauncherJob(service);
+    auto job = new KIO::ApplicationLauncherJob(service);
     connect(job, &KJob::finished, this, [this, service](KJob *job) {
         if (job->error()) {
             Q_EMIT backend()->passiveMessage(i18n("Failed to start '%1': %2", service->name(), job->errorString()));
@@ -534,11 +545,6 @@ void FlatpakResource::setBundledIcon(const QPixmap &pixmap)
     m_bundledIcon = pixmap;
 }
 
-void FlatpakResource::setCommit(const QString &commit)
-{
-    m_commit = commit;
-}
-
 void FlatpakResource::setDownloadSize(quint64 size)
 {
     m_downloadSize = size;
@@ -609,7 +615,7 @@ void FlatpakResource::setState(AbstractResource::State state, bool shouldEmit)
     if (m_state != state) {
         m_state = state;
 
-        if (shouldEmit && qobject_cast<FlatpakBackend *>(backend())->isTracked(this)) {
+        if (shouldEmit && backend()->isTracked(this)) {
             Q_EMIT stateChanged();
         }
     }
@@ -666,15 +672,16 @@ QDate FlatpakResource::releaseDate() const
 
 QString FlatpakResource::sourceIcon() const
 {
-    const auto sourceItem = qobject_cast<FlatpakBackend *>(backend())->sources()->sourceById(origin());
+    const auto sourceItem = backend()->sources()->sourceById(origin());
     if (!sourceItem) {
-        qWarning() << "Could not find source " << origin();
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Could not find source " << origin();
         return QStringLiteral("flatpak-discover");
     }
 
     const auto iconUrl = sourceItem->data(FlatpakSourcesBackend::IconUrlRole).toString();
-    if (iconUrl.isEmpty())
+    if (iconUrl.isEmpty()) {
         return QStringLiteral("flatpak-discover");
+    }
     return iconUrl;
 }
 
@@ -714,7 +721,7 @@ QString FlatpakResource::versionString()
         return {};
     }
     if (isInstalled()) {
-        auto ref = qobject_cast<FlatpakBackend *>(backend())->getInstalledRefForApp(this);
+        auto ref = backend()->getInstalledRefForApp(this);
         if (ref) {
             version = QString::fromUtf8(flatpak_installed_ref_get_appdata_version(ref));
         }
@@ -749,31 +756,49 @@ QString translateSymbolicName(const QStringView &name)
 
 QString FlatpakResource::eolReason()
 {
-    if (!m_eolReason.has_value()) {
-        auto futureWatcher = new QFutureWatcher<FlatpakRemoteRef *>(this);
-        connect(futureWatcher, &QFutureWatcher<FlatpakRemoteRef *>::finished, this, [this, futureWatcher]() {
-            g_autoptr(FlatpakRemoteRef) rref = futureWatcher->result();
-            if (rref) {
-                m_eolReason = QString::fromUtf8(flatpak_remote_ref_get_eol(rref));
-                Q_EMIT eolReasonChanged();
-            }
-            futureWatcher->deleteLater();
-        });
+    if (!m_eolReason.has_value() && !m_eolReasonTask.has_value()) {
+        QPointer<FlatpakResource> self = this;
 
-        futureWatcher->setFuture(QtConcurrent::run(qobject_cast<FlatpakBackend *>(backend())->threadPool(),
-                                                   &FlatpakRunnables::findRemoteRef,
-                                                   this,
-                                                   qobject_cast<FlatpakBackend *>(backend())->cancellable()));
-        return {};
+        m_eolReasonTask = [](FlatpakResource *self) -> QCoro::Task<std::optional<QString>> {
+            QPointer<FlatpakResource> guard = self;
+            auto backend = qobject_cast<FlatpakBackend *>(self->backend());
+            auto cancellable = backend->cancellable();
+
+            g_autoptr(FlatpakRemoteRef) rref = co_await QtConcurrent::run(backend->threadPool(), &FlatpakRunnables::findRemoteRef, self, cancellable);
+
+            if (guard && !g_cancellable_is_cancelled(cancellable) && rref) {
+                co_return QString::fromUtf8(flatpak_remote_ref_get_eol(rref));
+            }
+
+            co_return std::nullopt;
+        }(this)
+                                                           .then([guard = QPointer(this)](std::optional<QString> reason) {
+                                                               if (!guard) {
+                                                                   return;
+                                                               }
+                                                               guard->m_eolReasonTask.reset();
+                                                               if (reason.has_value()) {
+                                                                   guard->setEolReason(reason.value());
+                                                               }
+                                                           });
     }
     return m_eolReason.value_or(QString());
 }
 
+void FlatpakResource::setEolReason(const QString &reason)
+{
+    if (m_eolReason != reason) {
+        m_eolReason = reason;
+        Q_EMIT eolReasonChanged();
+    }
+}
+
 QString createHtmlList(const QStringList &itemList)
 {
     QString str = QStringLiteral("<ul>");
-    for (const QString &itemText : std::as_const(itemList))
-      str += QStringLiteral("<li>%1</li>").arg(itemText.toHtmlEscaped());
+    for (const QString &itemText : std::as_const(itemList)) {
+        str += QStringLiteral("<li>%1</li>").arg(itemText.toHtmlEscaped());
+    }
     str += QStringLiteral("</ul>");
     return str;
 }
@@ -959,8 +984,9 @@ void FlatpakResource::loadPermissions()
         if (sessionBusGroup.exists()) {
             const QStringList busList = sessionBusGroup.keyList();
             brief = i18n("Session Bus Access");
-            description = i18n("Can communicate with other applications and processes in the same desktop session using the following communication protocols: %1",
-                           createHtmlList(busList));
+            description =
+                i18n("Can communicate with other applications and processes in the same desktop session using the following communication protocols: %1",
+                     createHtmlList(busList));
             m_permissions.append(FlatpakPermission(brief, description, "plugins-symbolic"_L1));
         }
     }
@@ -995,18 +1021,24 @@ void FlatpakResource::clearUserData()
 {
     const auto location = dataLocation();
     if (location.isEmpty()) {
-        qWarning() << "Failed find location for" << name();
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed find location for" << name();
         return;
     }
 
     if (!QDir(location).removeRecursively()) {
-        qWarning() << "Failed to remove location" << location;
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to remove location" << location;
     }
     Q_EMIT hasDataChanged();
 }
 
 int FlatpakResource::versionCompare(FlatpakResource *resource) const
 {
+    // This is getting called from QML, we better sanitize the input
+    if (!resource) {
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Cannot compare with a null resource";
+        return -1;
+    }
+
     const QString other = resource->availableVersion();
     return AppStream::Utils::vercmpSimple(availableVersion(), other);
 }
@@ -1030,3 +1062,25 @@ uint FlatpakResource::contentRatingMinimumAge() const
 {
     return AppStreamUtils::contentRatingMinimumAge(m_appdata);
 }
+
+QStringList FlatpakResource::topObjects() const
+{
+    return s_topObjects;
+}
+
+QStringList FlatpakResource::bottomObjects() const
+{
+    return s_bottomObjects;
+}
+
+void FlatpakResource::addRefToUpdate(const QByteArray &toUpdate)
+{
+    m_toUpdate += toUpdate;
+}
+
+void FlatpakResource::clearToUpdate()
+{
+    m_toUpdate.clear();
+}
+
+#include "moc_FlatpakResource.cpp"
