@@ -62,7 +62,7 @@ QDebug operator<<(QDebug debug, const QSnapdSlot *slot)
     return debug;
 }
 
-const QStringList SnapResource::m_topObjects({QStringLiteral("qrc:/qml/PermissionsButton.qml")
+const QStringList SnapResource::s_topObjects({QStringLiteral("qrc:/qml/PermissionsButton.qml")
 #ifdef SNAP_CHANNELS
                                                   ,
                                               QStringLiteral("qrc:/qml/ChannelsButton.qml")
@@ -72,6 +72,8 @@ const QStringList SnapResource::m_topObjects({QStringLiteral("qrc:/qml/Permissio
 SnapResource::SnapResource(QSharedPointer<QSnapdSnap> snap, AbstractResource::State state, SnapBackend *backend)
     : AbstractResource(backend)
     , m_state(state)
+    , m_installedSize(0)
+    , m_downloadSize(0)
     , m_snap(snap)
 {
     setObjectName(snap->name());
@@ -100,8 +102,11 @@ QString SnapResource::comment()
 
 quint64 SnapResource::size()
 {
-    // return isInstalled() ? m_snap->installedSize() : m_snap->downloadSize();
-    return m_snap->downloadSize();
+    if (m_state == AbstractResource::Installed) {
+        return installedSize();
+    } else {
+        return downloadSize();
+    }
 }
 
 QVariant SnapResource::icon() const
@@ -298,9 +303,12 @@ void SnapResource::setSnap(const QSharedPointer<QSnapdSnap> &snap)
     if (m_snap == snap)
         return;
 
-    const bool newSize = m_snap->installedSize() != snap->installedSize() || m_snap->downloadSize() != snap->downloadSize();
+    const auto oldSize = size();
     m_snap = snap;
-    if (newSize)
+    updateSizes();
+    const auto newSize = size();
+
+    if (newSize != oldSize)
         Q_EMIT sizeChanged();
 
     Q_EMIT newSnap();
@@ -421,6 +429,11 @@ QString SnapResource::appstreamId() const
     return ids.isEmpty() ? QLatin1String("io.snapcraft.") + m_snap->name() + QLatin1Char('-') + m_snap->id() : ids.first();
 }
 
+QStringList SnapResource::topObjects() const
+{
+    return s_topObjects;
+}
+
 QString SnapResource::channel() const
 {
 #ifdef SNAP_PUBLISHER
@@ -463,6 +476,27 @@ void SnapResource::setChannel(const QString &channelName)
 #endif
 }
 
+quint64 SnapResource::installedSize() const
+{
+    return m_installedSize;
+}
+
+quint64 SnapResource::downloadSize() const
+{
+    return m_downloadSize;
+}
+
+void SnapResource::updateSizes()
+{
+    if (m_snap->installedSize() > 0) {
+        m_installedSize = m_snap->installedSize();
+    }
+
+    if (m_snap->downloadSize() > 0) {
+        m_downloadSize = m_snap->downloadSize();
+    }
+}
+
 void SnapResource::refreshSnap()
 {
     auto request = client()->find(QSnapdClient::FindFlag::MatchName, m_snap->name());
@@ -535,3 +569,4 @@ QObject *SnapResource::channels(QObject *parent)
 }
 
 #include "SnapResource.moc"
+#include "moc_SnapResource.cpp"
